@@ -68,6 +68,48 @@ class Document(treedocument.TreeDocumentMixin, document.Document):
             builder.rebuild(text)
 
 
+def find(name=None, *, filename=None, mimetype=None, contents=None):
+    """Find a root lexicon, either by language name, or by filename, mimetype
+    and/or contents.
+
+    If you specify a name, tries to find the language with that name, ignoring
+    the other arguments.
+
+    If you don't specify a name, but instead one or more of the other (keyword)
+    arguments, tries to find the language based on filename, mimetype or
+    contents.
+
+    If a language is found, returns the root lexicon. If no language could be
+    found, None is returned (which can also be used as root lexicon, resulting
+    in an empty token tree).
+
+    Examples::
+
+        >>> import parce
+        >>> parce.find("xml")
+        Xml.root
+        >>> parce.find(contents='{"key": 123;}')
+        Json.root
+        >>> parce.find(filename="style.css")
+        Css.root
+
+    This function uses the :mod:`~parce.registry` module and by default it
+    finds all bundled languages. See the module's documentation to find out how
+    to add your own languages to a registry.
+
+    """
+    from . import registry
+    if name:
+        lexicon_name = registry.find(name)
+    else:
+        for lexicon_name in registry.suggest(filename, mimetype, contents):
+            break
+        else:
+            return
+    if lexicon_name:
+        return registry.root_lexicon(lexicon_name)
+
+
 def root(root_lexicon, text):
     """Return the root context of the tree structure of all tokens from text."""
     return treebuilder.build_tree(root_lexicon, text)
@@ -183,9 +225,11 @@ def bytext(predicate, *itemlists):
 
 def ifgroup(n, itemlist, else_itemlist=()):
     r"""Return a :class:`~parce.rule.MatchItem` that yields ``itemlist`` if
-    group n in the match is not empty.
+    group n in the match is not None.
 
-    If group ``n`` in the match object is empty, ``else_itemlist`` is yielded.
+    If group ``n`` in the match object is None, ``else_itemlist`` is yielded.
+    A match group is None when the group was optional and did not participate
+    in the match.
 
     An example rule::
 
@@ -199,7 +243,7 @@ def ifgroup(n, itemlist, else_itemlist=()):
     (See also :func:`bygroup`.)
 
     """
-    predicate = lambda m: not m.group(m.lastindex + n)
+    predicate = lambda m: m.group(m.lastindex + n) is None
     return bymatch(predicate, itemlist, else_itemlist)
 
 
@@ -255,6 +299,10 @@ def _get_sequences_map(pairs, default):
     sequences = []
     itemlists = []
     all_items = set()
+    try:
+        pairs = pairs.items()   # succeeds if it's a dict
+    except AttributeError:
+        pass
     for s, i in pairs:
         s = frozenset(set(s) - all_items)
         all_items |= s
@@ -340,6 +388,9 @@ def mapgroup(n, dictionary, default=()):
 
     If the dict does not contain the key, the default value is yielded.
 
+    If you have an optional match group, use None as key value to select
+    the case the group was not present in the match.
+
     """
     get, itemlists = _get_items_map(dictionary, default)
     def predicate(m):
@@ -347,6 +398,17 @@ def mapgroup(n, dictionary, default=()):
     return bymatch(predicate, *itemlists)
 
 
+def maparg(dictionary, default=()):
+    r"""Return a :class:`~parce.rule.PredicateArgItem` that yields the itemlist
+    from the dictionary, using the current lexicon argument as key.
+
+    If the dict does not contain the key, the default value is yielded.
+
+    """
+    predicate, itemlists = _get_items_map(dictionary, default)
+    return byarg(predicate, *itemlists)
+
+
 def bygroup(*actions):
     r"""Return a :class:`~parce.action.SubgroupAction` that yields tokens for
     each subgroup in a regular expression.
