@@ -68,7 +68,7 @@ class FormatCache:
     def __init__(self, theme, factory, add_window=True):
         """Caches conversions from TextFormat to something else."""
         if add_window:
-            window = theme.window()
+            window = theme.baseformat()
             self.window = factory(window)
             self._format = lambda action: factory(window + theme.textformat(action))
         else:
@@ -85,8 +85,8 @@ class FormatContext:
     """A FormatContext is used during a formatting job.
 
     It maintains the state needed to format using a MetaTheme. The only API
-    used is the ``switch_theme()`` context manager method, which is called by
-    the ``tokens()`` method of MetaTheme.
+    used is the ``push()`` and ``pop()`` method, which are called by the
+    ``tokens()`` method of MetaTheme.
 
     The default Formatter uses the ``window`` property and ``format`` method
     to get the window style for the current theme and the factory that turns
@@ -94,23 +94,30 @@ class FormatContext:
 
     """
     def __init__(self, formatter):
+        self._stack = []
         self._formatter = formatter
-        c = self.cache = formatter.format_cache(formatter.theme())
-        self.window = c.window
-        self.format = c.format
+        self._current_theme = None
+        self._set_theme(formatter.theme())
 
-    @contextlib.contextmanager
-    def switch_theme(self, theme):
-        old = self.cache
-        c = self.cache = self._formatter.format_cache(theme)
-        self.window = c.window
-        self.format = c.format
-        try:
-            yield
-        finally:
-            self.cache = old
-            self.window = old.window
-            self.format = old.format
+    def _switch_language(self, language):
+        self._set_theme(self._formatter.theme().get_theme(language))
+
+    def _set_theme(self, theme):
+        if theme is not self._current_theme:
+            self._current_theme = theme
+            c = self.cache = self._formatter.format_cache(theme)
+            self.window = c.window
+            self.format = c.format
+
+    def push(self, language):
+        self._stack.append(language)
+        if len(self._stack) < 2 or language is not self._stack[-2]:
+            self._switch_language(language)
+
+    def pop(self):
+        language = self._stack.pop()
+        if len(self._stack) > 0 and language is not self._stack[-1]:
+            self._switch_language(self._stack[-1])
 
 
 class Formatter:
@@ -124,7 +131,6 @@ class Formatter:
         self._lock = threading.Lock()   # lock for FormatCaches
         self._theme = theme
         self._factory = factory or (lambda f: f)
-        self._current_theme = theme
         self._caches = weakref.WeakKeyDictionary()
         self._caches[theme] = c = FormatCache(theme, factory, False)
 
@@ -133,19 +139,9 @@ class Formatter:
         return self._theme
 
     @util.cached_method
-    def window(self, state="default"):
-        """Return our textformat for the window or encompassing DIV."""
-        return self._factory(self._theme.window(state))
-
-    @util.cached_method
-    def selection(self, state="default"):
-        """Return our textformat for the selected text."""
-        return self._factory(self._theme.selection(state))
-
-    @util.cached_method
-    def currentline(self, state="default"):
+    def baseformat(self, role="window", state="default"):
         """Return our textformat for the current line."""
-        return self._factory(self._theme.currentline(state))
+        return self._factory(self._theme.baseformat(role, state))
 
     def format_cache(self, theme):
         """Return a FormatCache for the Theme.
@@ -167,7 +163,7 @@ class Formatter:
                                     theme, self._factory, add_window)
                 return c
 
-    def format_ranges(self, slices):
+    def format_ranges(self, tree, start=0, end=None):
         """Yield FormatRange(pos, end, format) three-tuples.
 
         The ``format`` is the value returned by Theme.textformat() for the
@@ -178,15 +174,15 @@ class Formatter:
         """
         c = FormatContext(self)
         def stream():
-            end = sys.maxsize
-            for t in self._theme.tokens(c.switch_theme, slices):
-                if t.pos > end and c.window is not None:
+            prev_end = sys.maxsize
+            for t in self._theme.tokens(c, tree, start, end):
+                if t.pos > prev_end and c.window is not None:
                     # if a sub-language is active, draw its background
-                    yield end, t.pos, c.window
+                    yield prev_end, t.pos, c.window
                 f = c.format(t.action)
                 if f is not None:
                     yield t.pos, t.end, f
-                end = t.end
+                prev_end = t.end
         yield from util.merge_adjacent(stream(), FormatRange)
 
 
