@@ -46,6 +46,7 @@ RE_CSS_AT_KEYWORD = r"@" + RE_CSS_IDENTIFIER
 # match either 8, 6, 4 or 3 hex digits
 RE_HEX_COLOR = r"#[0-9a-fA-F]{3}(?:[0-9a-fA-F]{5}|[0-9a-fA-F]{3}|[0-9a-fA-F]?)"
 
+
 class Css(Language):
     @lexicon
     def root(cls):
@@ -60,7 +61,8 @@ class Css(Language):
 
     @lexicon
     def prelude(cls):
-        yield r"\{", Delimiter, -1, cls.rule
+        yield r"\{", Bracket, -1, cls.rule
+        yield r"(?=</)", None, -1   # back off if HTML </style> tag follows...
         yield from cls.selectors()
 
     @classmethod
@@ -78,7 +80,7 @@ class Css(Language):
     def selector(cls):
         yield r"\*", Keyword    # "any" element
         yield r"\|", Keyword    # css selector namespace prefix separator
-        yield r"#", Name.Identifier, cls.id_selector
+        yield r"#", Name.Identifier.Definition, cls.id_selector
         yield r"\.(?!\d)", Keyword, cls.class_selector
         yield r"::", Keyword, cls.pseudo_element
         yield r":", Keyword, cls.pseudo_class
@@ -95,7 +97,7 @@ class Css(Language):
     @lexicon
     def rule(cls):
         """Declarations of a qualified rule between { and }."""
-        yield r"\}", Delimiter, -1
+        yield r"\}", Bracket, -1
         yield from cls.inline()
 
     @lexicon
@@ -120,7 +122,7 @@ class Css(Language):
         yield r'"', String, cls.dqstring
         yield r"'", String, cls.sqstring
         yield r"/\*", Comment, cls.comment
-        yield r"\{", Delimiter, cls.rule
+        yield r"\{", Bracket, cls.rule
         yield RE_CSS_NUMBER, Number, cls.unit
         yield RE_HEX_COLOR, Literal.Color
         yield r"(url)(\()", bygroup(Name, Delimiter), cls.url_function
@@ -151,7 +153,9 @@ class Css(Language):
     @lexicon
     def property(cls):
         """A CSS property."""
-        yield from cls.identifier_common(Name.Property)
+        from .css_words import CSS3_ALL_PROPERTIES
+        yield from cls.identifier_common(
+            ifmember(CSS3_ALL_PROPERTIES, Name.Property.Definition, Name.Property))
 
     @lexicon
     def attribute(cls):
@@ -161,7 +165,7 @@ class Css(Language):
     @lexicon
     def id_selector(cls):
         """#id"""
-        yield from cls.identifier_common(Name.Identifier)
+        yield from cls.identifier_common(Name.Identifier.Definition)
 
     @lexicon
     def class_selector(cls):
@@ -179,24 +183,24 @@ class Css(Language):
     def pseudo_class(cls):
         """Things like :first-child etc."""
         yield r"\(", Delimiter, -1, cls.selector_list
-        yield from cls.identifier_common(Name.Pseudo.Class)
+        yield from cls.identifier_common(Name.Class.Pseudo)
 
     @lexicon
     def pseudo_element(cls):
         """Things like ::first-letter etc."""
-        yield from cls.identifier_common(Name.Pseudo.Tag)
+        yield from cls.identifier_common(Name.Tag.Pseudo)
 
     # --------------------- @-rule ------------------------
     @lexicon
     def atrule(cls):
         """Contents following '@'."""
-        yield r"\{", Delimiter, cls.atrule_block
+        yield r"\{", Bracket, cls.atrule_block
         yield from cls.atrule_common()
 
     @lexicon
     def atrule_nested(cls):
         """An atrule that has nested toplevel contents (@media, etc.)"""
-        yield r"\{", Delimiter, cls.atrule_nested_block
+        yield r"\{", Bracket, cls.atrule_nested_block
         yield from cls.atrule_common()
 
     @lexicon
@@ -208,13 +212,13 @@ class Css(Language):
     @lexicon
     def atrule_block(cls):
         """a { } block from an @-rule."""
-        yield r"\}", Delimiter, -2  # immediately leave the atrule context
+        yield r"\}", Bracket, -2  # immediately leave the atrule context
         yield from cls.inline()
 
     @lexicon
     def atrule_nested_block(cls):
         """a { } block from @media, @document or @supports."""
-        yield r"\}", Delimiter, -2  # immediately leave the atrule_nested context
+        yield r"\}", Bracket, -2  # immediately leave the atrule_nested context
         yield from cls.toplevel()
 
     @classmethod
@@ -223,6 +227,7 @@ class Css(Language):
         yield r";", Delimiter, -1
         yield r":", Keyword, cls.pseudo_class
         yield from cls.common()
+        yield r'(?=</)', None, -1   # leave atrule when </style tag follows
 
     @lexicon
     def identifier(cls):
@@ -230,7 +235,7 @@ class Css(Language):
         from .css_words import CSS3_NAMED_COLORS
         yield r"\(", Delimiter, -1, cls.function
         yield RE_CSS_ESCAPE, Escape
-        yield r"[\w-]+", ifmember(CSS3_NAMED_COLORS, Literal.Color, Name)
+        yield r"[\w-]+", ifmember(CSS3_NAMED_COLORS, Literal.Color, Name.Symbol)
         yield default_target, -1
 
     @lexicon
@@ -269,7 +274,7 @@ class Css(Language):
         yield default_action, String
         yield RE_CSS_ESCAPE, String.Escape
         yield r"\\\n", String.Escape
-        yield r"\n", Error, -1
+        yield r"\n", Invalid, -1
 
     @lexicon
     def comment(cls):
