@@ -47,8 +47,17 @@ class Language:
 
     @classmethod
     def comment_common(cls):
-        """Highlights TODO, XXX and TEMP inside comments using Comment.Alert."""
-        yield r"\b(XXX|TODO|TEMP)\b", parce.Comment.Alert
+        """Provides subtle highlighting within comments.
+
+        The default implementation highlight TODO, XXX and TEMP
+        using Comment.Alert, and highlights URLs and email addresses with the
+        Comment.Url and Comment.Email action respectively.
+        Most bundled languages use this method for their comment lexicons.
+
+        """
+        yield r'\b\w(?:[._%+-]?\w+)*@\w(?:[._-]?\w+)*\b', parce.Comment.Email
+        yield r'(?:https?|ftp):/(?:[\w._~:?/#-]+|\([\w._~:?/#-]*\))+', parce.Comment.Url
+        yield r"\b(ALERT|BUG|FIXME|TEMP|TODO|XXX+)\b", parce.Comment.Alert
         yield parce.default_action, parce.Comment
 
 
@@ -121,6 +130,7 @@ def get_languages(name):
     mod = importlib.import_module(modname)
     for name, obj in mod.__dict__.items():
         if (isinstance(obj, type) and issubclass(obj, parce.Language)
+               and not obj.__name__.startswith('_')
                and obj is not parce.Language and obj.__module__ == modname):
             yield obj
 
