@@ -94,25 +94,36 @@ from . import themes
 from . import util
 
 
-TextFormatRange = collections.namedtuple("TextFormatRange", "pos end textformat theme")
-
-
 class Theme:
     """A Theme maps a StandardAction to a TextFormat with CSS properties."""
-    def __init__(self, filename):
-        """Instantiate Theme from a CSS file."""
+    def __init__(self, filename="", stylesheet=""):
+        """Instantiate Theme from a CSS file or text.
+
+        If the ``stylesheet`` text is given, it is used as the stylesheet. Only
+        if the text is empty, the filename is used to read the stylesheet from.
+
+        """
         self._filename = filename
+        self._css_text = stylesheet
         self.TextFormat = TextFormat
 
     @classmethod
     def byname(cls, name="default"):
-        """Create Theme by name, that should reside in the themes/ directory."""
+        """Create Theme by name.
+
+        The name is a CSS file in the themes/ directory, without the ".css"
+        extension.
+
+        """
         return cls(themes.filename(name))
 
     @util.cached_property
     def _stylesheet(self):
         """Load and cache the StyleSheet."""
-        return css.StyleSheet.from_file(self._filename)
+        if self._css_text or not self._filename:
+            return css.StyleSheet.from_text(self._css_text, self._filename)
+        else:
+            return css.StyleSheet.from_file(self._filename)
 
     @util.cached_property
     def style(self):
@@ -124,41 +135,42 @@ class Theme:
         return self._stylesheet.filenames()
 
     @util.cached_method
-    def window(self, state="default"):
-        """Return the default textformat.
+    def baseformat(self, role="window", state="default"):
+        """Return a TextFormat for a specific role and a state.
 
-        This is intended to be used for the editor window or encompassing DIV
-        element. The state argument may be "default", "focus", or "disabled".
+        The role may be any string that maps to a CSS class in the theme
+        CSS file that is available there together with the 'parce' class.
+
+        The following roles are recognized and used by parce, but you may
+        also define your own roles in your (applications') theme CSS files:
+
+        ``"window"``
+            The TextFormat for the editor window or the encompassing DIV when
+            formatting HTML. Corresponds to the "parce" CSS class alone in the
+            theme file. You can set color, background and, if desired, font
+            preferences.
+        ``"selection"``
+            The TextFormat to use for selected text. Uses the ``::selection``
+            pseudo element.
+        ``"current-line"``
+            The TextFormat for the current line. If you use it, set only the
+            *background* color in your theme file.
+        ``"trialing-whitespace"``
+            The TextFormat (use only the *background*) to highlight trialing
+            whitespace, if desired.
+        ``"eol-marker"``
+            The *color* to draw a "end-of-line" marker with, if desired
+
+        The state argument may be "default", "focus", or "disabled", and
+        reflects the state of the user interface the style variant is used for.
 
         """
-        e = css.Element(class_="parce")
-        if state in ("focus", "disabled"):
-            e.pseudo_classes = [state]
-        return self.TextFormat(self.style.select_element(e).properties())
-
-    @util.cached_method
-    def selection(self, state="default"):
-        """Return the default textformat for selected text.
-
-        This is intended to be used for the editor window or encompassing DIV
-        element. The state argument may be "default", "focus", or "disabled".
-
-        """
-        e = css.Element(class_="parce", pseudo_elements=["selection"])
-        if state in ("focus", "disabled"):
-            e.pseudo_classes = [state]
-        return self.TextFormat(self.style.select_element(e).properties())
-
-    @util.cached_method
-    def currentline(self, state="default"):
-        """Return the default textformat for the current line.
-
-        This is intended to be used for the editor window or encompassing DIV
-        element, probably only the background color. The state argument may be
-        "default", "focus", or "disabled".
-
-        """
-        e = css.Element(class_="parce current-line")
+        if role == "window":
+            e = css.Element(class_="parce")
+        elif role == "selection":
+            e = css.Element(class_="parce", pseudo_elements=["selection"])
+        else:
+            e = css.Element(class_="parce {}".format(role))
         if state in ("focus", "disabled"):
             e.pseudo_classes = [state]
         return self.TextFormat(self.style.select_element(e).properties())
@@ -166,28 +178,21 @@ class Theme:
     @util.cached_method
     def textformat(self, action):
         """Return the TextFormat for the specified action."""
-        classes = repr(action).lower().split('.')
-        return self.TextFormat(self.style.select_class(*classes).properties())
+        class_ = repr(action).lower().replace('.', ' ')
+        e = css.Element(class_=class_, parent=css.Element(class_="parce"))
+        return self.TextFormat(self.style.select_element(e).properties())
 
-    def tokens(self, theme_context, slices):
-        """Yield tokens from the slices, calling ``theme_context`` if necessary.
+    def tokens(self, theme_context, tree, start=0, end=None):
+        """Yield tokens from ``tree.tokens_range(start, end)``.
 
-        The ``slices`` argument is an iterable of (context, slice) tuples.
-        such as returned by Context.slices() and Context.context_slices().
-        When a theme changes, ``theme_context()`` is called with the theme
-        for a new context with that theme (``theme_context(theme)`` should
-        return a context manager). In Theme, the ``theme_context`` argument
-        is unused.
+        If a context is entered, ``theme_context.push(language)`` is called
+        with the language of the context's lexicon. If the context is left,
+        ``theme_context.pop()`` is called.
+
+        In Theme, the ``theme_context`` argument is unused.
 
         """
-        def events(nodes):
-            for n in nodes:
-                if n.is_token:
-                    yield n
-                else:
-                    yield from events(n)
-        for context, slice_ in slices:
-            yield from events(context[slice_])
+        yield from tree.tokens_range(start, end)
 
 
 class MetaTheme:
@@ -232,52 +237,48 @@ class MetaTheme:
         """Return the ``add_window`` value that was set when adding this sub-theme."""
         return self._add_window.get(theme, False)
 
-    def window(self, state="default"):
-        """Return the window TextFormat for the state of the default theme."""
-        return self._theme.window(state)
-
-    def selection(self, state="default"):
-        """Return the selection TextFormat for the state of the default theme."""
-        return self._theme.selection(state)
-
-    def currentline(self, state="default"):
-        """Return the currentline TextFormat for the state of the default theme."""
-        return self._theme.currentline(state)
+    def baseformat(self, role="window", state="default"):
+        """Return the base TextFormat for the rold and state of the default theme."""
+        return self._theme.baseformat(state)
 
     def textformat(self, action):
         """Return the TextFormat for the specified action of the default theme."""
         return self._theme.textformat(action)
 
-    def tokens(self, theme_context, slices):
-        """Yield tokens, calling theme_context() to set the correct theme.
+    def tokens(self, theme_context, tree, start=0, end=None):
+        """Yield tokens from ``tree.tokens_range(start, end)``.
 
-        The ``slices`` argument is an iterable of (context, slice) tuples.
-        such as returned by Context.slices() and Context.context_slices().
-        When a theme changes, ``theme_context()`` is called with the theme
-        for a new context with that theme (``theme_context(theme)`` should
-        return a context manager).
+        If a context is entered, ``theme_context.push(language)`` is called
+        with the language of the context's lexicon. If the context is left,
+        ``theme_context.pop()`` is called.
+
+        The theme_context can switch the theme, based on the current language.
 
         """
-        def events(nodes, current_language, current_theme):
-            lang = nodes[0].parent.lexicon.language
-            if lang is not current_language:
-                theme = self.get_theme(lang)
-                if theme is not current_theme:
-                    with theme_context(theme):
-                        for n in nodes:
-                            if n.is_token:
-                                yield n
-                            else:
-                                yield from events(n, lang, theme)
-                    return
-            for n in nodes:
-                if n.is_token:
-                    yield n
-                else:
-                    yield from events(n, current_language, current_theme)
-        lang = None
-        for context, slice_ in slices:
-            yield from events(context[slice_], lang, self)
+        for context, slice_ in tree.context_slices(start, end):
+            theme_context.push(context.lexicon.language)
+            for n in context[slice_]:
+                stack = []
+                i = 0
+                while True:
+                    for i in range(i, len(n)):
+                        m = n[i]
+                        if m.is_token:
+                            yield m
+                        else:
+                            stack.append(i)
+                            i = 0
+                            n = m
+                            theme_context.push(n.lexicon.language)
+                            break
+                    else:
+                        if stack:
+                            theme_context.pop()
+                            n = n.parent
+                            i = stack.pop() + 1
+                        else:
+                            break
+            theme_context.pop()
 
 
 class TextFormat:
