@@ -223,7 +223,7 @@ class TreeBuilder:
         offset = added - removed
 
         if not root_lexicon:
-            return BuildResult(Context(root_lexicon, None), start, start + added, 0, None)
+            return BuildResult(Context(root_lexicon, None), start, start + added, 0, [])
 
         # If there remains text after the modified part,
         # we try to reuse the old tokens
@@ -310,9 +310,12 @@ class TreeBuilder:
                         start = c.start
                         if c.root_lexicon != False:
                             root_lexicon = c.root_lexicon
+                            if not root_lexicon:
+                                return BuildResult(Context(root_lexicon, None), 0, len(text), 0, [])
                             start = 0
                             tail = False
                             tree = None
+
                         elif tail:
                             # reuse old tail?
                             new_tail_pos = start + c.added
@@ -332,15 +335,14 @@ class TreeBuilder:
                     if tokens[-1].pos > peek:
                         # call peek with a copy of the current tree.
                         copied_tree = tree.copy()
-                        if copied_tree:
-                            # remove the first empty context
-                            t = copied_tree[0]
-                            while t and t.is_context:
-                                t = t[0]
-                            while t.is_context and not t:
-                                t = t.parent
-                                del t[0]
-                            self.peek(copied_tree)
+                        # remove the first empty context
+                        t = copied_tree[0]
+                        while t and t.is_context:
+                            t = t[0]
+                        while t.is_context and not t:
+                            t = t.parent
+                            del t[0]
+                        self.peek(lowest_start, copied_tree)
                         peek = 0
             else:
                 # we ran till the end, also return the open lexicons
@@ -348,7 +350,7 @@ class TreeBuilder:
         raise RuntimeError("shouldn't come here")
 
     def replace_tree(self, result):
-        """Modify the tree using the result from build_new().
+        """Modify the tree using the result from :meth:`build_new_tree`.
 
         In most types of GUI applications, this method should be called in the
         main (GUI) thread.
@@ -559,7 +561,7 @@ class TreeBuilder:
         """
         pass
 
-    def peek(self, tree):
+    def peek(self, start, tree):
         """This is called from :meth:`build_new_tree` with a sneak preview tree.
 
         This can be used to get a small tree before the new tree is built
@@ -597,7 +599,7 @@ class TreeBuilder:
         pass
 
     def process_started(self):
-        """Called when ``start()`` has been called to update the tree.
+        """Called at the start ot the tree building process.
 
         Does nothing by default.
 
