@@ -20,6 +20,9 @@
 
 """
 Various utility classes and functions.
+
+This module only depends on the Python standard library.
+
 """
 
 import re
@@ -143,7 +146,7 @@ def cached_method(func):
     """Wrap a method and caches its return value.
 
     The method argument tuple should be hashable. Keyword arguments are not
-    supported. The cache is tread-safe. Does not keep a reference to the
+    supported. The cache is thread-safe. Does not keep a reference to the
     instance.
 
     """
@@ -256,3 +259,27 @@ def split_list(l, separator):
 def quote(s):
     """Like repr, but return s with double quotes, escaping " and \\."""
     return '"' + re.sub(r'([\\"])', r'\\\1', s) + '"'
+
+
+def tokens(nodes):
+    """Helper to yield tokens from the iterable of nodes."""
+    for n in nodes:
+        if n.is_token:
+            yield n
+        else:
+            yield from n.tokens()
+
+
+def tokens_bw(nodes):
+    """Helper to yield tokens from the iterable in backward direction.
+
+    Make sure nodes is already in backward direction.
+
+    """
+    for n in nodes:
+        if n.is_token:
+            yield n
+        else:
+            yield from n.tokens_bw()
+
+
